library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity PS2Controller_tb is
	--  Port ( );
end PS2Controller_tb;

architecture Behavioral of PS2Controller_tb is
	component PS2Controller is
		port (
			clk   : in  std_logic;
			rst   : in  std_logic;
			kclk  : in  std_logic;
			kdata : in  std_logic;
			data  : out std_logic_vector (8 downto 0);
			eot   : out std_logic);
	end component;

	-- Vhodni signali, ki UUT povezujejo z generatorjem draljajev
	signal clk, rst     : std_logic := '0';
	signal kclk, kdata  : std_logic := '1';

	-- Izhodni signali, ki UUT povezujejo z osciloskopom
	signal eot          : std_logic;
	signal data         : std_logic_vector(8 downto 0);

	-- Konstante
	constant CLK_PERIOD : time                          := 10 ns;
	-- Podatki s tipkovnice za simulacijo 
	-- Simuliramo pritisk tipke ENTER, ki ima kodo 0x5A = 0101 1010.
	-- Poleg podatkovnega bajta vkljucimo tudi start, stop in paritetni bit in dobimo 11 bitov.
	-- Podatke pisemo v smeri, ki je obratna, kot ce bi jih pisali glede na cas.
	-- STOP PAR B7 B6 ...... B0 START
	--  1    0  0  1  01 101 0    0  
	constant SIM_DATA   : std_logic_vector(10 downto 0) := "10010110100";

begin

	UUT : PS2Controller
	port map(
		clk              => clk,
		kclk             => kclk,
		kdata            => kdata,
		rst              => rst,
		eot              => eot,
		data(8 downto 0) => data(8 downto 0)
	);

	-- Generator draljajev 
	-- Ura 
	CLK_STIMULUS : process
	begin
		clk <= not clk;
		wait for CLK_PERIOD/2;
	end process;

	-- Ostali signali 
	OTH_STIMULI : process
	begin
		-- Reset
		rst   <= '1';
		kclk  <= '1';
		kdata <= '1';
		wait for CLK_PERIOD * 3;

		-- Mirovanje
		rst   <= '0';
		kclk  <= '1';
		kdata <= '1';
		wait for CLK_PERIOD * 3;

		-- Zamik ure tipkovnice, da (namenoma) ne bo sinhrona z notranjo uro
		wait for CLK_PERIOD/3;

		-- Pritisk tipke ENTER
		for i in 0 to 10 loop
			kdata <= SIM_DATA(i);
			wait for CLK_PERIOD * 6;
			kclk <= '0';
			wait for CLK_PERIOD * 6;
			kclk <= '1';
		end loop;

		-- Mirovanje
		kclk  <= '1';
		kdata <= '1';
		wait;
	end process;
end Behavioral;